@@ -53,6 +53,8 @@ import_complete = re.compile(_import_complete + "?$")
 
 class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
 
+    MIMETYPES = ['', 'py', 'pyc']
+
     def __init__(self, *args, **kwargs):
         super(PythonCodeCompletionModel, self).__init__(*args, **kwargs)
         self.resultList = []
@@ -79,18 +81,24 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
         line_start = word.start().line()
         line_end = word.end().line()
         self.resultList = []
+        self.invocationType = invocationType
+        path = unicode(view.document().url().path())
         if line_start != line_end:
             return
+        if not path.split(".")[-1] in self.MIMETYPES:
+            return
         doc = view.document()
         line = unicode(doc.line(line_start))
         if not line:
             return
-        line = line.strip()
+        line = self._parse_line(line)
+        line_rough = line
         if 'from' in line or 'import' in line:
             is_auto = self.autoCompleteImport(view, word, line)
+        line = self._get_expression_last_expression(line)
         if not is_auto and line:
             is_auto = self.autoCompleteDynamic(view, word, line)
-        if not is_auto and line:
+        if not is_auto and line and line_rough and not '.' in line_rough:
             is_auto = self.autoCompleteInThisFile(view, word, line)
 
     #http://api.kde.org/4.5-api/kdelibs-apidocs/interfaces/ktexteditor/html/classKTextEditor_1_1CodeCompletionModel.html#3bd60270a94fe2001891651b5332d42b
@@ -118,18 +126,32 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                 pass
         elif index.column() == KTextEditor.CodeCompletionModel.Icon:
             if role == Qt.DecorationRole:
-                return KIcon(item["icon"]).pixmap(QSize(16, 16))
+                return QVariant(KIcon(item["icon"]).pixmap(QSize(16, 16)))
         elif index.column() == KTextEditor.CodeCompletionModel.Arguments:
             item_args = item.get("args", None)
             if role == Qt.DisplayRole and item_args:
-                return item_args
+                return QVariant(item_args)
         elif index.column() == KTextEditor.CodeCompletionModel.Postfix:
             item_description = item.get("description", None)
             if role == Qt.DisplayRole and item_description:
-                return item_description
+                return QVariant(item_description)
         return QVariant()
 
     def executeCompletionItem(self, doc, word, row):
+        raw, col = word.start().position()
+        line = unicode(doc.line(raw))
+        line = self._parse_line(line)
+        t = self.resultList[row].get('type', None)
+        args = self.resultList[row].get('args', None).strip()
+        text = self.resultList[row].get('text', None)
+        if not "from" in line and not "import" in line and \
+           t in ['function', 'class']:
+            if args == '()':
+                doc.replaceText(word, '%s()' % text)
+                return
+            else:
+                doc.replaceText(word, '%s(' % text)
+                return
         return super(PythonCodeCompletionModel, self).executeCompletionItem(doc, word, row)
 
     def autoCompleteImport(self, view, word, line):
@@ -167,21 +189,42 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
             line = line[:last_dot]
         except ValueError:
             pass
-        doc = view.document()
-        text_list = unicode(doc.text()).split("\n")
-        raw, column = word.start().position()
-        del text_list[raw]
-        text = '\n'.join(text_list)
+        text = self._parse_text(view, word, line)
         return self.getDynamic(text, line)
 
     def autoCompleteInThisFile(self, view, word, line):
+        text = self._parse_text(view, word, line)
+        return self.getTextInfo(text, self.resultList)
+
+    def _parse_line(self, line):
+        line = line.strip()
+        if "'" in line or '"' in line:
+            return line
+        if ";" in line:
+            return self._parse_line(line.split(";")[-1])
+        return line
+
+    def _get_expression_last_expression(self, line):
+        operators = ["=", " ", "[", "]", "(", ")", "{", "}", ":", ">", "<",
+                     "+", "-", "*", "/", "%", "and", "or"]
+        opmax = max(operators, key=lambda e: line.rfind(e))
+        opmax_index = line.rfind(opmax)
+        if line.find(opmax) != -1:
+            line = line[opmax_index + 1:]
+        return line.strip()
+
+    def _parse_text(self, view, word, line):
         doc = view.document()
         text_list = unicode(doc.text()).split("\n")
         raw, column = word.start().position()
         line = text_list[raw]
-        del text_list[raw]
+        if ";" in line and not "'" in line and not '"' in line:
+            text_list[raw] = ';'.join(text_list[raw].split(";")[:-1])
+        else:
+            del text_list[raw]
         text = '\n'.join(text_list)
-        return self.getTextInfo(text, self.resultList, line)
+        line = line.strip()
+        return text
 
     @classmethod
     def getTopLevelModules(cls):
@@ -193,30 +236,18 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                 module_name = None
                 if filename.endswith(".py"):
                     module_name = filename.split(os.sep)[-1][:-3]
+                    icon = 'module'
                 elif os.path.isdir(filename) and os.path.exists(filename + os.sep + "__init__.py"):
                     module_name = filename.split(os.sep)[-1]
-                module = cls.createItemAutoComplete(module_name, 'module')
+                    icon = 'package'
+                if not module_name:
+                    continue
+                module = cls.createItemAutoComplete(module_name, icon)
                 if module and not module in modules:
                     modules.append(module)
                     modules_path[module_name] = [filename]
         return modules
 
-    def getModuleSmart(self, module_name, submodules, submodules_undone=None):
-        module_dir = modules_path[module_name][0]
-        submodules_undone = submodules_undone or []
-        submodules_str = os.sep.join(submodules)
-        module_dir = "%s%s%s" % (module_dir, os.sep, submodules_str)
-        att_dir = os.sep.join(module_dir.split(os.sep)[:-1])
-        att_module = module_dir.split(os.sep)[-1].replace('.py', '').replace('.pyc', '')
-        importer = pkgutil.get_importer(att_dir)
-        module = importer.find_module(att_module)
-        if module:
-            return (module, submodules_undone)
-        elif submodules:
-            submodules_undone.append(submodules[-1])
-            return self.getModuleSmart(module_name, submodules[:-1], submodules_undone)
-        return (None, None)
-
     def getSubmodules(self, module_name, submodules=None,
                        attributes=True):
         module_dir = modules_path[module_name][0]
@@ -226,7 +257,8 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
             module_dir = "%s%s%s" % (module_dir, os.sep, submodules)
         modules = []
         for loader, module_name, is_pkg in pkgutil.walk_packages([module_dir]):
-            module = self.createItemAutoComplete(module_name, 'module')
+            icon = is_pkg and 'package' or 'module'
+            module = self.createItemAutoComplete(module_name, icon)
             modules.append(module)
         if attributes:
             att_dir = os.sep.join(module_dir.split(os.sep)[:-1])
@@ -237,6 +269,23 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                 self.getTextInfo(module.get_source(), modules)
         return sorted(modules)
 
+    def getModuleSmart(self, module_name, submodules, submodules_undone=None):
+        module_dir = modules_path[module_name][0]
+        submodules_undone = submodules_undone or []
+        submodules_str = os.sep.join(submodules)
+        if submodules_str:
+            module_dir = "%s%s%s" % (module_dir, os.sep, submodules_str)
+        att_dir = os.sep.join(module_dir.split(os.sep)[:-1])
+        att_module = module_dir.split(os.sep)[-1].replace('.py', '').replace('.pyc', '')
+        importer = pkgutil.get_importer(att_dir)
+        module = importer.find_module(att_module)
+        if module:
+            return (module, submodules_undone)
+        elif submodules:
+            submodules_undone.append(submodules[-1])
+            return self.getModuleSmart(module_name, submodules[:-1], submodules_undone)
+        return (None, submodules_undone)
+
     def getDynamic(self, text, code_line, text_info=True):
         try:
             code = parse(text)
@@ -263,8 +312,9 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                 module_done, submodules_undone = self.getModuleSmart(module, submodules)
                 if not submodules_undone:
                     autocompletion_submodules = self.getSubmodules(module, submodules, True)
-                    if autocompletion_submodules:
-                        self.resultList = list(set(self.resultList.extend(autocompletion_submodules)))
+                    for autocompletion_submodule in autocompletion_submodules:
+                        if autocompletion_submodule not in self.resultList:
+                            self.resultList.append(autocompletion_submodule)
                 submodules_undone.reverse()
                 line = '.'.join(submodules_undone)
                 text = module_done.get_source()
@@ -273,8 +323,7 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                 if text_info:
                     return self.getTextInfo(text, self.resultList)
         except SyntaxError, e:
-            kate.gui.popup('There was a syntax error in this file',
-                            2, icon='dialog-warning', minTextWidth=200)
+            self.treatment_exception(e)
         return False
 
     @classmethod
@@ -297,18 +346,22 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
     def createItemAutoComplete(cls, text,
                                     icon='unknown',
                                     args=None,
-                                    description=None):
-        icon_converter = {'module': 'code-block',
+                                    description=None,
+                                    place='code'):
+        icon_converter = {'package': 'code-block',
+                          'module': 'code-context',
                           'unknown': 'unknown',
                           'constant': 'code-variable',
                           'class': 'code-class',
                           'function': 'code-function'}
         max_description = 50
         if description and len(description) > max_description:
+            description = description.strip()
             description = '%s...' % description[:max_description]
         return {'text': text,
                 'icon': icon_converter[icon],
                 'args': args or '',
+                'type': icon,
                 'description': description or ''}
 
     def getTextInfo(self, text, list_autocomplete, line=None):
@@ -318,15 +371,21 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
             modules = code_walk.modules
             constans = modules['CONSTANTS']
             for constant in constans:
-                list_autocomplete.append(self.treatment_pysmell_const(constant))
+                item = self.treatment_pysmell_const(constant)
+                if not item in list_autocomplete:
+                    list_autocomplete.append(item)
 
             functions = modules['FUNCTIONS']
             for func in functions:
-                list_autocomplete.append(self.treatment_pysmell_func(func))
+                item = self.treatment_pysmell_func(func)
+                if not item in list_autocomplete:
+                    list_autocomplete.append(item)
 
             classes = modules['CLASSES']
             for cls in classes.items():
-                list_autocomplete.append(self.treatment_pysmell_cls(cls))
+                item = self.treatment_pysmell_cls(cls)
+                if not item in list_autocomplete:
+                    list_autocomplete.append(item)
 
             is_auto = len(list_autocomplete) > 0
             if not is_auto and line:
@@ -338,10 +397,26 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
                                         text_info=False)
             return is_auto
         except SyntaxError, e:
-            kate.gui.popup('There was a syntax error in this file',
-                            2, icon='dialog-warning', minTextWidth=200)
+            self.treatment_exception(e)
         return False
 
+    def treatment_exception(self, e):
+        if self.invocationType == KTextEditor.CodeCompletionModel.AutomaticInvocation:
+            return
+        f = e.filename or ''
+        text = e.text
+        line = e.lineno
+        message = 'There was a syntax error in this file:'
+        if f:
+            message = '%s\n  * file: %s' %(message, f)
+        if text:
+            message = '%s\n  * text: %s' %(message, text)
+        if line:
+            message = '%s\n  * line: %s' %(message, line)
+        kate.gui.popup(message,
+                       2, icon='dialog-warning', minTextWidth=200)
+
+
     def treatment_pysmell_const(self, constant):
         constant = constant.replace(PYSMELL_PREFIX, '')
         return self.createItemAutoComplete(constant, 'constant')
@@ -370,11 +445,17 @@ class PythonCodeCompletionModel(KTextEditor.CodeCompletionModel):
 
     def treatment_pysmell_into_cls(self, cls, resultList):
         for m in cls['methods']:
-            resultList.append(self.treatment_pysmell_func(m))
+            item = self.treatment_pysmell_func(m)
+            if not item in resultList:
+                resultList.append(item)
         for m in cls['constructor']:
-            resultList.append(self.treatment_pysmell_func(m))
+            item = self.treatment_pysmell_func(m)
+            if not item in resultList:
+                resultList.append(item)
         for m in cls['properties']:
-            resultList.append(self.treatment_pysmell_const(m))
+            item = self.treatment_pysmell_const(m)
+            if not item in resultList:
+                resultList.append(item)
 
 
 def createSignalAutocompleteDocument(view, *args, **kwargs):
